#ifndef _OUTPUT_H_
#define _OUTPUT_H_

#include <string>
#include "decode.h"
#include "NewCString.h"
using namespace std;


#pragma pack(push, 1)
class __declspec(dllexport) OutputTree {
	int lines;
	Tree *cb;
	Tree *b;
	int  csize;
	int  base_offset, level;
	bool *show;
	bool processLines;
	char summary[1000];

public:
	struct {
		bool  IsInitialized;
		unsigned short FrameControl;
		unsigned short Duration;
		unsigned char Mac1[6];
		unsigned char Mac2[6];
		unsigned char Mac3[6];
		unsigned short SeqControl;
		unsigned char Mac4[6];
	} ieee80211;

	struct{
		bool  IsInitialized;
		unsigned char dst_mac[6], src_mac[6];
		unsigned short proto;
	} ethernet;

	struct{
		bool  IsInitialized;
		unsigned char ver_len;
		unsigned char tos;
		unsigned short total_length;
		unsigned short id;
		unsigned short flags_offset;
		unsigned char ttl;
		unsigned char proto;
		unsigned short csum;
		unsigned char src_ip[4], dst_ip[4];
		unsigned char *after_ip_hdr;
	} ip;

	struct{
		bool  IsInitialized;
		unsigned short src_port, dst_port;
		unsigned short length;
		unsigned short csum;
	} udp;

	struct{
		bool  IsInitialized;
		unsigned short src_port, dst_port;
		unsigned int seq, ack;
		unsigned char off;
		unsigned char flags;
		unsigned short window;
		unsigned short csum;
		unsigned short urgent;
	} tcp;

	unsigned char *ieee80211Hdr;
	unsigned char *etherHdr;
	unsigned char *ipHdr;
	unsigned char *udpHdr;
	unsigned char *tcpHdr;

	unsigned long ip_csum;
	unsigned char wifiType;
	unsigned char wifiSubtype;

	OutputTree(Tree *const buffer, const int buf_size);
	~OutputTree();

  void AddOffset(const int delta); 
  void SetOffset(const int offset); 
  int  GetOffset(void); 

	// Add new line
	int PutString(const int level, const int type, const int offset, const int size, char *const name, char *const value);
	int PutString(const int level, const int type, const int offset, const int size, char *const name, NewCString& value);
	int PutString(const int level, const int type, const int offset, const int size, NewCString& name, char *const value);
	int PutString(const int level, const int type, const int offset, const int size, NewCString& name, NewCString& value);
	int PutStringComment(const int level, const int type, const int offset, const int size, char *const name, char *const value, char *const cmt);
	int PutInt(const int level, const int type, const int offset, const int size, char *const name, const int num);
	int PutIntComment(const int level, const int type, const int offset, const int size, char *const name, const int num, char *const cmt);
	int PutIntComment(const int level, const int type, const int offset, const int size, char *const name, const int num, NewCString& cmt);
	int PutUint64(const int level, const int type, const int offset, const int size, char *const name, const unsigned __int64 num);
	int PutUint64Comment(const int level, const int type, const int offset, const int size, char *const name, const unsigned __int64 num, char *const cmt);
	int PutULong(const int level, const int type, const int offset, const int size, char *const name, const unsigned long num);
	int PutULongComment(const int level, const int type, const int offset, const int size, char *const name, const unsigned long num, char *const cmt);
	int PutLong(const int level, const int type, const int offset, const int size, char *const name, const signed long num);
	int PutLongComment(const int level, const int type, const int offset, const int size, char *const name, const signed long num, char *const cmt);
	int PutFloat(const int level, const int type, const int offset, const int size, char *const name, float const num);
	int PutFloatComment(const int level, const int type, const int offset, const int size, char *const name, float const num, char *const cmt);
	int PutDouble(const int level, const int type, const int offset, const int size, char *const name, double const num);
	int PutDoubleComment(const int level, const int type, const int offset, const int size, char *const name, double const num, char *const cmt);
	int PutIPv4Addr(const int level, const int type, const int offset, const int size, char *const name, unsigned char *const ip);
	int PutIPv6Addr(const int level, const int type, const int offset, const int size, char *const name, unsigned char *const ip);
	int PutIPXnet(const int level, const int type, const int offset, const int size, char *const name, unsigned char *const ipx_net);
	int PutIPXnode(const int level, const int type, const int offset, const int size, char *const name, unsigned char *const ipx_node);
	int PutMAC(const int level, const int type, const int offset, const int size, char *const name, unsigned char *const MAC);
	int PutUnkAddr(const int level, const int type, const int offset, const int size, char *const name, unsigned char *const addr, int addr_size);
	int PutBinary(const int level, const int type, const int offset, const int size, char *const name, const unsigned int num, const int len);
	int PutBinaryComment(const int level, const int type, const int offset, const int size, char *const name, const unsigned int num, const int len, char *const cmt);
	int PutDate(const int level, const int type, const int offset, const int size, char *const name, int day, int month, int year);
	int PutTime(const int level, const int type, const int offset, const int size, char *const name, int hour, int min, int sec);
	int PutHexString(const int level, const int type, const int offset, const int size, char *const name, unsigned char *const str, const unsigned short len);
	int PutHexStringComment(const int level, const int type, const int offset, const int size, char *const name, unsigned char *const str, const unsigned short len, char *const cmt);

	// Update added line
	void UpdateString(const int pos, char *const value);
	void UpdateString(const int pos, NewCString& value);
	void UpdateString(const int pos, string value);
	void UpdateName(const int pos, char *const name);
	void UpdateSize(const int pos, const int size);
	void AddSize(const int pos, const int size);
	void UpdateOffset(const int pos, const int offset);
	void UpdateLevel(const int new_level);

	// Format functions
	void const FormatInt(char *const buffer, const int type, const int num, const int size);

	// Get lines
	int const GetLines() {
		return lines;
	}

	bool const IsNull() {
		return csize == 0;
	}

	// Tree scanning
	int GetNode(int pos, char *name, char *value);
	int GetSubNode(int pos, char *name, char *value);

	// Node show/hide
	void HideNode(int pos);
	void ShowNode(int pos);

	// Process hides/shows
	void ProcessHides(void);

	// Get Ethernet/IP/TCP/UDP pointers
	unsigned char *GetIeee80211HdrPtr() {
		return ieee80211Hdr;
	}

	unsigned char *Get8023HdrPtr() {
		return etherHdr;
	}

	unsigned char *GetIpHdrPtr() {
		return ipHdr;
	}

	unsigned char *GetUdpHdrPtr() {
		return udpHdr;

	}

	unsigned char *GetTcpHdrPtr() {
		return tcpHdr;
	}

	void setSummary(char *protocol, char *str);
	void setSummary(NewCString& protocol, NewCString& str);
  char* getSummary();
};
#pragma pack(pop)

#endif //_OUTPUT_H_
